@@ -11,6 +11,8 @@
 
 #include <SDL3_image/SDL_image.h>
 
+#include "IMG_utils.h"
+
 // Used because CGDataProviderCreate became deprecated in 10.5
 #include <AvailabilityMacros.h>
 #include <TargetConditionals.h>
@@ -44,7 +46,6 @@ static size_t MyProviderGetBytesCallback(void* userdata, void* quartz_buffer, si
 // so you can clean up any resources.
 static void MyProviderReleaseInfoCallback(void* userdata)
 {
-    (void)userdata;
     // What should I put here?
     // I think the user and SDL_IOStream controls closing, so I don't do anything.
 }
@@ -150,16 +151,14 @@ static CGImageRef CreateCGImageFromCGImageSource(CGImageSourceRef image_source)
 {
     CGImageRef image_ref = NULL;
 
-    if(NULL == image_source)
-    {
+    if (!image_source) {
         return NULL;
     }
 
     // Get the first item in the image source (some image formats may
     // contain multiple items).
     image_ref = CGImageSourceCreateImageAtIndex(image_source, 0, NULL);
-    if(NULL == image_ref)
-    {
+    if (!image_ref) {
         SDL_SetError("CGImageSourceCreateImageAtIndex() failed");
     }
     return image_ref;
@@ -345,14 +344,25 @@ static SDL_Surface* Create_SDL_Surface_From_CGImage_Index(CGImageRef image_ref)
 
     return surface;
 }
-static SDL_Surface* Create_SDL_Surface_From_CGImage(CGImageRef image_ref)
+
+static SDL_Surface *Create_SDL_Surface_From_CGImage(CGImageRef image_ref, CFDictionaryRef properties)
 {
+    SDL_Surface *surface;
     CGColorSpaceRef color_space = CGImageGetColorSpace(image_ref);
     if (CGColorSpaceGetModel(color_space) == kCGColorSpaceModelIndexed) {
-        return Create_SDL_Surface_From_CGImage_Index(image_ref);
+        surface = Create_SDL_Surface_From_CGImage_Index(image_ref);
     } else {
-        return Create_SDL_Surface_From_CGImage_RGB(image_ref);
+        surface = Create_SDL_Surface_From_CGImage_RGB(image_ref);
     }
+    if (surface && properties) {
+        CFNumberRef numval;
+        if (CFDictionaryGetValueIfPresent(properties, kCGImagePropertyOrientation, (const void **)&numval)) {
+            CGImagePropertyOrientation orientation;
+            CFNumberGetValue(numval, kCFNumberSInt32Type, &orientation);
+            surface = IMG_ApplyOrientation(surface, orientation);
+        }
+    }
+    return surface;
 }
 
 
@@ -426,15 +436,6 @@ bool IMG_isJPG(SDL_IOStream *src)
 
 #endif /* JPG_USES_IMAGEIO */
 
-#ifdef PNG_USES_IMAGEIO
-
-bool IMG_isPNG(SDL_IOStream *src)
-{
-    return Internal_isType(src, kUTTypePNG);
-}
-
-#endif /* PNG_USES_IMAGEIO */
-
 // This isn't a public API function. Apple seems to be able to identify tga's.
 bool IMG_isTGA(SDL_IOStream *src)
 {
@@ -446,49 +447,49 @@ bool IMG_isTIF(SDL_IOStream *src)
     return Internal_isType(src, kUTTypeTIFF);
 }
 
-static SDL_Surface *LoadImageFromIOStream (SDL_IOStream *rw_ops, CFStringRef uti_string_hint)
+static SDL_Surface *LoadImageFromIOStream(SDL_IOStream *rw_ops, CFStringRef uti_string_hint)
 {
+    SDL_Surface *surface = NULL;
     CFDictionaryRef hint_dictionary = CreateHintDictionary(uti_string_hint);
     CGImageSourceRef image_source = CreateCGImageSourceFromIOStream(rw_ops, hint_dictionary);
 
-    if (hint_dictionary != NULL)
+    if (hint_dictionary) {
         CFRelease(hint_dictionary);
-
-    if (NULL == image_source) {
-        return NULL;
     }
 
-    CGImageRef image_ref = CreateCGImageFromCGImageSource(image_source);
-    CFRelease(image_source);
-
-    if (NULL == image_ref) {
-        return NULL;
+    if (image_source) {
+        CFDictionaryRef properties = CGImageSourceCopyPropertiesAtIndex(image_source, 0, NULL);
+        CGImageRef image_ref = CreateCGImageFromCGImageSource(image_source);
+        if (image_ref) {
+            surface = Create_SDL_Surface_From_CGImage(image_ref, properties);
+            CFRelease(image_ref);
+        }
+        if (properties) {
+            CFRelease(properties);
+        }
+        CFRelease(image_source);
     }
-    SDL_Surface *sdl_surface = Create_SDL_Surface_From_CGImage(image_ref);
-    CFRelease(image_ref);
-
-    return sdl_surface;
+    return surface;
 }
 
-static SDL_Surface* LoadImageFromFile (const char *file)
+static SDL_Surface *LoadImageFromFile(const char *file)
 {
-    CGImageSourceRef image_source = NULL;
+    SDL_Surface *surface = NULL;
+    CGImageSourceRef image_source = CreateCGImageSourceFromFile(file);
 
-    image_source = CreateCGImageSourceFromFile(file);
-
-    if (NULL == image_source) {
-        return NULL;
+    if (image_source) {
+        CFDictionaryRef properties = CGImageSourceCopyPropertiesAtIndex(image_source, 0, NULL);
+        CGImageRef image_ref = CreateCGImageFromCGImageSource(image_source);
+        if (image_ref) {
+            surface = Create_SDL_Surface_From_CGImage(image_ref, properties);
+            CFRelease(image_ref);
+        }
+        if (properties) {
+            CFRelease(properties);
+        }
+        CFRelease(image_source);
     }
-
-    CGImageRef image_ref = CreateCGImageFromCGImageSource(image_source);
-    CFRelease(image_source);
-
-    if (NULL == image_ref) {
-        return NULL;
-    }
-    SDL_Surface *sdl_surface = Create_SDL_Surface_From_CGImage(image_ref);
-    CFRelease(image_ref);
-    return sdl_surface;
+    return surface;
 }
 
 #ifdef BMP_USES_IMAGEIO
@@ -527,7 +528,7 @@ SDL_Surface* IMG_LoadJPG_IO (SDL_IOStream *src)
 
 #ifdef PNG_USES_IMAGEIO
 
-SDL_Surface* IMG_LoadPNG_IO (SDL_IOStream *src)
+SDL_Surface* IMG_LoadPNG_ImageIO (SDL_IOStream *src)
 {
     return LoadImageFromIOStream (src, kUTTypePNG);
 }
